@@ -3,13 +3,13 @@ package POE::Component::Server::SimpleHTTP;
 
 # Standard stuff to catch errors
 use strict qw(subs vars refs);				# Make sure we can't mess up
-use warnings FATAL => 'all';				# Enable warnings to catch errors
+use warnings;
 
 use vars qw($VERSION);
 
 # Initialize our version
 # $Revision: 1181 $
-$VERSION = '1.27_03';
+$VERSION = '1.53_01';
 
 # Import what we need from the POE namespace
 use POE;
@@ -21,6 +21,7 @@ use POE::Filter::Stream;
 
 # Other miscellaneous modules we need
 use Carp qw( croak );
+use Socket;
 
 # HTTP-related modules
 use HTTP::Date qw( time2str );
@@ -54,9 +55,10 @@ sub new {
 
 	# The options hash
 	my %opt = @_;
+	$opt{uc $_} = delete $opt{$_} for keys %opt; # Fix that pesky uppercase option stuff.
 
 	# Our own options
-	my ( $ALIAS, $ADDRESS, $PORT, $HOSTNAME, $HEADERS, $HANDLERS, $SSLKEYCERT, $LOGHANDLER, $ERRORHANDLER, $SETUPHANDLER );
+	my ( $ALIAS, $ADDRESS, $PORT, $HOSTNAME, $HEADERS, $HANDLERS, $SSLKEYCERT, $LOGHANDLER, $ERRORHANDLER, $SETUPHANDLER, $LOG2HANDLER, $PROXYMODE );
 
 	# You could say I should do this: $Stuff = delete $opt{'Stuff'}
 	# But, that kind of behavior is not defined, so I would not trust it...
@@ -70,7 +72,7 @@ sub new {
 
 			# Okay, pull in what is necessary
 			eval {
-				require POE::Component::SSLify; 
+				require POE::Component::SSLify;
 				import POE::Component::SSLify qw( SSLify_Options SSLify_GetSocket Server_SSLify SSLify_GetCipher );
 				SSLify_Options( @$SSLKEYCERT );
 			};
@@ -82,12 +84,14 @@ sub new {
 				# Force ourself to not use SSL
 				$SSLKEYCERT = undef;
 			}
-		} else {
+		} 
+		else {
 			if ( DEBUG ) {
 				warn 'The SSLKEYCERT option must be an array with exactly 2 elements in it!';
 			}
 		}
-	} else {
+	} 
+	else {
 		$SSLKEYCERT = undef;
 	}
 
@@ -101,7 +105,8 @@ sub new {
 	if ( exists $opt{'PORT'} and defined $opt{'PORT'} and length( $opt{'PORT'} ) ) {
 		$PORT = $opt{'PORT'};
 		delete $opt{'PORT'};
-	} else {
+	} 
+	else {
 		croak( 'PORT is required to create a new POE::Component::Server::SimpleHTTP instance!' );
 	}
 
@@ -118,7 +123,8 @@ sub new {
 	if ( exists $opt{'HOSTNAME'} and defined $opt{'HOSTNAME'} and length( $opt{'HOSTNAME'} ) ) {
 		$HOSTNAME = $opt{'HOSTNAME'};
 		delete $opt{'HOSTNAME'};
-	} else {
+	} 
+	else {
 		if ( DEBUG ) {
 			warn 'Using Sys::Hostname for HOSTNAME';
 		}
@@ -139,10 +145,12 @@ sub new {
 		if ( ref $opt{'HEADERS'} and ref( $opt{'HEADERS'} ) eq 'HASH' ) {
 			$HEADERS = $opt{'HEADERS'};
 			delete $opt{'HEADERS'};
-		} else {
+		} 
+		else {
 			croak( 'HEADERS must be a reference to a HASH!' );
 		}
-	} else {
+	} 
+	else {
 		# Set to none
 		$HEADERS = {};
 
@@ -158,10 +166,12 @@ sub new {
 		if ( ref $opt{'HANDLERS'} and ref( $opt{'HANDLERS'} ) eq 'ARRAY' ) {
 			$HANDLERS = $opt{'HANDLERS'};
 			delete $opt{'HANDLERS'};
-		} else {
+		} 
+		else {
 			croak( 'HANDLERS must be a reference to an ARRAY!' );
 		}
-	} else {
+	} 
+	else {
 		croak( 'HANDLERS is required to create a new POE::Component::Server::SimpleHTTP instance!' );
 	}
 
@@ -170,11 +180,12 @@ sub new {
 		# Make sure it is ref to an hash
 		if ( ref $opt{'ERRORHANDLER'} and ref( $opt{'ERRORHANDLER'} ) eq 'HASH' ) {
 			$ERRORHANDLER = delete $opt{'ERRORHANDLER'};
-			croak( 'ERRORHANDLER does not have a SESSION attribute' ) 
+			croak( 'ERRORHANDLER does not have a SESSION attribute' )
 			  unless $ERRORHANDLER->{'SESSION'};
-			croak( 'ERRORHANDLER does not have an EVENT attribute' ) 
+			croak( 'ERRORHANDLER does not have an EVENT attribute' )
 			  unless $ERRORHANDLER->{'EVENT'};
-		} else {
+		} 
+		else {
 			croak( 'ERRORHANDLER must be a reference to an HASH!' );
 		}
 	}
@@ -182,9 +193,9 @@ sub new {
 	if ( exists $opt{'LOGHANDLER'} and defined $opt{'LOGHANDLER'} ) {
 		if ( ref $opt{'LOGHANDLER'} and ref $opt{'LOGHANDLER'} eq 'HASH' ) {
 			$LOGHANDLER = delete $opt{'LOGHANDLER'};
-			croak( 'LOGHANDLER does not have a SESSION attribute' ) 
+			croak( 'LOGHANDLER does not have a SESSION attribute' )
 			  unless $LOGHANDLER->{'SESSION'};
-			croak( 'LOGHANDLER does not have an EVENT attribute' ) 
+			croak( 'LOGHANDLER does not have an EVENT attribute' )
 			  unless $LOGHANDLER->{'EVENT'};
 		}
 		else {
@@ -192,18 +203,38 @@ sub new {
 		}
 	}
 
+	if ( exists $opt{'LOG2HANDLER'} and defined $opt{'LOG2HANDLER'} ) {
+		if ( ref $opt{'LOG2HANDLER'} and ref $opt{'LOG2HANDLER'} eq 'HASH' ) {
+			$LOG2HANDLER = delete $opt{'LOG2HANDLER'};
+			croak( 'LOG2HANDLER does not have a SESSION attribute' )
+			  unless $LOG2HANDLER->{'SESSION'};
+			croak( 'LOG2HANDLER does not have an EVENT attribute' )
+			  unless $LOG2HANDLER->{'EVENT'};
+		}
+		else {
+			croak( 'LOG2HANDLER must be a reference to an HASH!' );
+		}
+	}
+
 	if ( exists $opt{'SETUPHANDLER'} and defined $opt{'SETUPHANDLER'} ) {
 		if ( ref $opt{'SETUPHANDLER'} and ref $opt{'SETUPHANDLER'} eq 'HASH' ) {
 			$SETUPHANDLER = delete $opt{'SETUPHANDLER'};
-			croak( 'SETUPHANDLER does not have a SESSION attribute' ) 
+			croak( 'SETUPHANDLER does not have a SESSION attribute' )
 			  unless $SETUPHANDLER->{'SESSION'};
-			croak( 'SETUPHANDLER does not have an EVENT attribute' ) 
+			croak( 'SETUPHANDLER does not have an EVENT attribute' )
 			  unless $SETUPHANDLER->{'EVENT'};
 		}
 		else {
 			croak( 'SETUPHANDLER must be a reference to an HASH!' );
 		}
 	}
+	
+	$PROXYMODE = $opt{'PROXYMODE'};
+
+    my $KEEPALIVE = 0;
+    if ( exists $opt{'KEEPALIVE'} ) {
+        $KEEPALIVE = delete $opt{'KEEPALIVE'};
+    }
 
 	# Anything left over is unrecognized
 	if ( DEBUG ) {
@@ -211,8 +242,8 @@ sub new {
 			croak( 'Unrecognized options were present in POE::Component::Server::SimpleHTTP->new -> ' . join( ', ', keys %opt ) );
 		}
 	}
-	
-	my $data = { 
+
+	my $data = {
 			'ALIAS'		   =>	$ALIAS,
 			'ADDRESS'	   =>	$ADDRESS,
 			'PORT'		   =>	$PORT,
@@ -223,8 +254,11 @@ sub new {
 			'RETRIES'	   =>	0,
 			'SSLKEYCERT'   =>	$SSLKEYCERT,
 			'LOGHANDLER'   =>	$LOGHANDLER,
+			'LOG2HANDLER'   =>	$LOG2HANDLER,
 			'SETUPHANDLER' =>	$SETUPHANDLER,
 			'ERRORHANDLER' =>	$ERRORHANDLER,
+			'PROXYMODE' => 		$PROXYMODE,
+            'KEEPALIVE'    =>   $KEEPALIVE
 	};
 
 	my $self = bless $data, $type;
@@ -237,7 +271,7 @@ sub new {
 			'_start'	=>	\&StartServer,
 			'_stop'		=>	\&FindRequestLeaks,
 			'_child'	=>	sub {},
-			
+
 		   # Register states
 		   'REGISTER' => \&Register,
 
@@ -260,12 +294,15 @@ sub new {
 
 			# Send output to connection!
 			'DONE'		=>	\&Request_Output,
-			
+
 			# Stream output to connection!
 			'STREAM'	   =>	\&Stream_Output,
 
 			# Kill the connection!
 			'CLOSE'		=>	\&Request_Close,
+
+            # Set close handler for a connection
+            'SETCLOSEHANDLER' => \&SetCloseHandler
 		},
 
 		# Set up the heap for ourself
@@ -303,22 +340,23 @@ sub FindRequestLeaks {
 
 # Starts the server!
 sub StartServer {
+	my $heap = $_[HEAP];
 	# Debug stuff
 	if ( DEBUG ) {
 		warn 'Starting up SimpleHTTP now';
 	}
 
-	$_[HEAP]->{SESSION_ID} = $_[SESSION]->ID();
+	$heap->{SESSION_ID} = $_[SESSION]->ID();
 
 	# Register an alias for ourself
-	$_[KERNEL]->alias_set( $_[HEAP]->{'ALIAS'} ) if $_[HEAP]->{'ALIAS'};
-	$_[KERNEL]->refcount_increment( $_[HEAP]->{SESSION_ID}, __PACKAGE__ ) unless $_[HEAP]->{'ALIAS'};
+	$poe_kernel->alias_set( $heap->{'ALIAS'} ) if $heap->{'ALIAS'};
+	$poe_kernel->refcount_increment( $heap->{SESSION_ID}, __PACKAGE__ ) unless $heap->{'ALIAS'};
 
 	# Massage the handlers!
-	MassageHandlers( $_[HEAP]->{'HANDLERS'} );
+	MassageHandlers( $heap->{'HANDLERS'} );
 
 	# Setup the wheel
-	$_[KERNEL]->yield( 'SetupListener' );
+	$poe_kernel->yield( 'SetupListener' );
 
 	# All done!
 	return 1;
@@ -343,7 +381,9 @@ sub StopServer {
 			# Alright, shutdown anyway
 
 			# Delete our alias
-			$_[KERNEL]->alias_remove( $_[HEAP]->{'ALIAS'} );
+			$_[KERNEL]->alias_remove( $_[HEAP]->{'ALIAS'} ) if $_[HEAP]->{'ALIAS'};
+			$_[KERNEL]->refcount_decrement( $_[HEAP]->{'SESSION_ID'}, __PACKAGE__ )
+				unless $_[HEAP]->{'ALIAS'};
 
 			# Debug stuff
 			if ( DEBUG ) {
@@ -356,21 +396,24 @@ sub StopServer {
 	}
 
 	# Forcibly close all sockets that are open
-	foreach my $conn ( keys %{ $_[HEAP]->{'REQUESTS'} } ) {
-		# Can't call method "shutdown_input" on an undefined value at
-		# /usr/lib/perl5/site_perl/5.8.2/POE/Component/Server/SimpleHTTP.pm line 323.
-		if ( defined $_[HEAP]->{'REQUESTS'}->{ $conn }->[0] and defined $_[HEAP]->{'REQUESTS'}->{ $conn }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
-			$_[HEAP]->{'REQUESTS'}->{ $conn }->[0]->shutdown_input;
-			$_[HEAP]->{'REQUESTS'}->{ $conn }->[0]->shutdown_output;
-		}
+    foreach my $S ( $_[HEAP]->{'REQUESTS'}, $_[HEAP]->{'CONNECTIONS'} ) {
+        foreach my $conn ( keys %$S ) {
+    		# Can't call method "shutdown_input" on an undefined value at
+	    	# /usr/lib/perl5/site_perl/5.8.2/POE/Component/Server/SimpleHTTP.pm line 323.
+		    if ( defined $S->{ $conn }->[0] and
+                    defined $S->{ $conn }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
+    			$S->{ $conn }->[0]->shutdown_input;
+	    		$S->{ $conn }->[0]->shutdown_output;
+		    }
+            # Delete this request
+		    delete $S->{ $conn };
+	    }
+    }
 
-		# Delete this request
-		delete $_[HEAP]->{'REQUESTS'}->{ $conn };
-	}
 
 	# Delete our alias
 	$_[KERNEL]->alias_remove( $_[HEAP]->{'ALIAS'} ) if $_[HEAP]->{'ALIAS'};
-	$_[KERNEL]->refcount_decrement( $_[HEAP]->{'SESSION_ID'}, __PACKAGE__ ) 
+	$_[KERNEL]->refcount_decrement( $_[HEAP]->{'SESSION_ID'}, __PACKAGE__ )
 		unless $_[HEAP]->{'ALIAS'};
 
 	# Debug stuff
@@ -385,32 +428,36 @@ sub StopServer {
 # Sets up the SocketFactory wheel :)
 sub SetupListener {
 	# Debug stuff
+	my $heap = $_[HEAP];
 	if ( DEBUG ) {
 		warn 'Creating SocketFactory wheel now';
 	}
 
 	# Check if we should set up the wheel
-	if ( $_[HEAP]->{'RETRIES'} == MAX_RETRIES ) {
+	if ( $heap->{'RETRIES'} == MAX_RETRIES ) {
 		die 'POE::Component::Server::SimpleHTTP tried ' . MAX_RETRIES . ' times to create a Wheel and is giving up...';
-	} else {
+	} 
+	else {
 		# Increment the retry count if we did not get 'NOINC' as an argument
 		if ( ! defined $_[ARG0] ) {
 			# Increment the retries count
-			$_[HEAP]->{'RETRIES'}++;
+			$heap->{'RETRIES'}++;
 		}
 
 		# Create our own SocketFactory Wheel :)
-		$_[HEAP]->{'SOCKETFACTORY'} = POE::Wheel::SocketFactory->new(
-			'BindPort'	=>	$_[HEAP]->{'PORT'},
-			'BindAddress'	=>	$_[HEAP]->{'ADDRESS'},
+		$heap->{'SOCKETFACTORY'} = POE::Wheel::SocketFactory->new(
+			'BindPort'	=>	$heap->{'PORT'},
+			'BindAddress'	=>	$heap->{'ADDRESS'},
 			'Reuse'		=>	'yes',
 			'SuccessEvent'	=>	'Got_Connection',
 			'FailureEvent'	=>	'ListenerError',
 		);
-	        $_[KERNEL]->post(
-	   		$_[HEAP]->{'SETUPHANDLER'}->{'SESSION'},
-	   		$_[HEAP]->{'SETUPHANDLER'}->{'EVENT'},
-		) if $_[HEAP]->{'SETUPHANDLER'};
+		my ($port,$address) = sockaddr_in( $heap->{'SOCKETFACTORY'}->getsockname );
+		$heap->{'PORT'} = $port if $heap->{'PORT'} == 0;
+	        $poe_kernel->post(
+	   		$heap->{'SETUPHANDLER'}->{'SESSION'},
+	   		$heap->{'SETUPHANDLER'}->{'EVENT'}, $port, $address,
+		) if $heap->{'SETUPHANDLER'};
 	}
 
 	# Success!
@@ -519,6 +566,9 @@ sub MassageHandlers {
 	while ( $count < scalar( @$handler ) ) {
 		# Must be ref to hash
 		if ( ref $handler->[ $count ] and ref( $handler->[ $count ] ) eq 'HASH' ) {
+			# Make sure all the keys are uppercase
+			$handler->[ $count ]->{uc $_} = delete $handler->[ $count ]->{$_}
+				for keys %{ $handler->[ $count ] };
 			# Make sure it got the 3 parts necessary
 			if ( ! exists $handler->[ $count ]->{'SESSION'} or ! defined $handler->[ $count ]->{'SESSION'} ) {
 				croak( "HANDLER number $count does not have a SESSION argument!" );
@@ -542,11 +592,13 @@ sub MassageHandlers {
 			# Check for errors
 			if ( $@ ) {
 				croak( "HANDLER number $count has a malformed DIR -> $@" );
-			} else {
+			} 
+			else {
 				# Store it!
 				$handler->[ $count ]->{'RE'} = $regex;
 			}
-		} else {
+		} 
+		else {
 			croak( "HANDLER number $count is not a reference to a HASH!" );
 		}
 
@@ -584,6 +636,12 @@ sub Got_Connection {
 		'ErrorEvent'	=>	'Got_Error',
 	);
 
+    if( DEBUG and keys %{ $_[HEAP]->{CONNECTIONS} } ) {
+        # use Data::Dumper;
+        warn "conn id=", $wheel->ID, " [",
+            join( ', ', keys %{ $_[HEAP]->{CONNECTIONS} }), "]";
+    }
+
 	# Save this wheel!
 	# 0 = wheel, 1 = Output done?, 2 = SimpleHTTP::Response object
 	$_[HEAP]->{'REQUESTS'}->{ $wheel->ID } = [ $wheel, 0, undef ];
@@ -602,6 +660,18 @@ sub Got_Input {
 	# ARG0 = HTTP::Request object, ARG1 = Wheel ID
 	my( $request, $id ) = @_[ ARG0, ARG1 ];
 
+    my $connection;
+
+    # Was this request Keep-Alive?
+    if( $_[HEAP]->{'CONNECTIONS'}->{ $id } ) {
+        my $c = delete $_[HEAP]->{'CONNECTIONS'}->{ $id };
+        $_[HEAP]->{'REQUESTS'}->{ $id } = [ $c->[0], 0, undef ];
+        $connection = $c->[1];
+        if ( DEBUG ) {
+            warn "Keep-alive id=$id next request...";
+        }
+    }
+
 	# Quick check to see if the socket died already...
 	# Initially reported by Tim Wood
 	if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $id }->[0] or ! defined $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
@@ -617,7 +687,21 @@ sub Got_Input {
 		return;
 	}
 
-	# The HTTP::Response object, the path
+	if( $connection ) {
+        # connection was kept-alive
+	}
+	# Directly access POE::Wheel::ReadWrite's HANDLE_INPUT -> to get the socket itself
+	# Hmm, if we are SSL, then have to do an extra step!
+	elsif( defined $_[HEAP]->{'SSLKEYCERT'} ) {
+		$connection =
+			POE::Component::Server::SimpleHTTP::Connection->new( SSLify_GetSocket( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) );
+	}
+	else {
+		$connection =
+			POE::Component::Server::SimpleHTTP::Connection->new( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] );
+	}
+
+    # The HTTP::Response object, the path
 	my ( $response, $path, $malformed_req );
 
 	# Check if it is HTTP::Request or Response
@@ -629,7 +713,7 @@ sub Got_Input {
 		# Make the request nothing
 		$response = $request;
 		$request = undef;
-		
+
 		# Mark that this is a malformed request
 		$malformed_req = 1;
 
@@ -637,44 +721,35 @@ sub Got_Input {
 		bless( $response, 'POE::Component::Server::SimpleHTTP::Response' );
 		$response->{'WHEEL_ID'} = $id;
 
-		# Directly access POE::Wheel::ReadWrite's HANDLE_INPUT -> to get the socket itself
-		# Hmm, if we are SSL, then have to do an extra step!
-		if ( defined $_[HEAP]->{'SSLKEYCERT'} ) {
-			$response->{'CONNECTION'} = POE::Component::Server::SimpleHTTP::Connection->new( SSLify_GetSocket( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) );
-		} else {
-			$response->{'CONNECTION'} = POE::Component::Server::SimpleHTTP::Connection->new( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] );
-		}
+        $response->{'CONNECTION'} = $connection;
 
 		# Set the path to an empty string
 		$path = '';
-	} else {
-		# Add stuff it needs!
-		my $uri = $request->uri;
-		$uri->scheme( 'http' );
-		$uri->host( $_[HEAP]->{'HOSTNAME'} );
-		$uri->port( $_[HEAP]->{'PORT'} );
+	} 
+	else {
+		unless ( $_[HEAP]->{'PROXYMODE'} ) {
+			# Add stuff it needs!
+			my $uri = $request->uri;
+			$uri->scheme( 'http' );
+			$uri->host( $_[HEAP]->{'HOSTNAME'} );
+			$uri->port( $_[HEAP]->{'PORT'} );
 
-		# Get the path
-		$path = $uri->path();
-		if ( ! defined $path or $path eq '' ) {
-			# Make it the default handler
-			$path = '/';
+			# Get the path
+			$path = $uri->path();
+			if ( ! defined $path or $path eq '' ) {
+				# Make it the default handler
+				$path = '/';
+			}
+		}
+		else {
+			# We're in PROXYMODE set the path to the full URI
+			$path = $request->uri->as_string();
 		}
 
 		# Get the response
-		# Directly access POE::Wheel::ReadWrite's HANDLE_INPUT -> to get the socket itself
-		# Hmm, if we are SSL, then have to do an extra step!
-		if ( defined $_[HEAP]->{'SSLKEYCERT'} ) {
-			$response = POE::Component::Server::SimpleHTTP::Response->new(
-				$id,
-				POE::Component::Server::SimpleHTTP::Connection->new( SSLify_GetSocket( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) )
+        $response = POE::Component::Server::SimpleHTTP::Response->new(
+				$id, $connection
 			);
-		} else {
-			$response = POE::Component::Server::SimpleHTTP::Response->new(
-				$id,
-				POE::Component::Server::SimpleHTTP::Connection->new( $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] )
-			);
-		}
 
 		# Stuff the default headers
 		if ( keys( %{ $_[HEAP]->{'HEADERS'} } ) != 0 ) {
@@ -695,7 +770,8 @@ sub Got_Input {
 
 		# All done!
 		return;
-	} else {
+	} 
+	else {
 		# If we used SSL, turn on the flag!
 		if ( defined $_[HEAP]->{'SSLKEYCERT'} ) {
 			$response->{'CONNECTION'}->{'SSLified'} = 1;
@@ -708,33 +784,43 @@ sub Got_Input {
 	# Add this response to the wheel
 	$_[HEAP]->{'REQUESTS'}->{ $id }->[2] = $response;
 	$_[HEAP]->{'REQUESTS'}->{ $id }->[3] = $request;
-	
+    $response->{'CONNECTION'}->{ id } = $id;
+
 	# If they have a log handler registered, send out the needed information
 	# TODO if we received a malformed request, we will not have a request object
 	# We need to figure out what we're doing because they can't always expect to have
 	# a request object, or should we keep it from being ?undef'd?
-	$_[KERNEL]->post(
-	   $_[HEAP]->{'LOGHANDLER'}->{'SESSION'},
-	   $_[HEAP]->{'LOGHANDLER'}->{'EVENT'},
-	   $request,
-	   $response->connection->remote_ip(),
-	) if $_[HEAP]->{'LOGHANDLER'};
-	
-	# Warn if we had a problem dispatching to the log handler above
-	warn("I had a problem posting to event '",
-	     $_[HEAP]->{'LOGHANDLER'}->{'EVENT'},
-	     "' of the log handler alias '",
-	     $_[HEAP]->{'LOGHANDLE'}->{'SESSION'},
-          "'. As reported by Kernel: '$!', perhaps the alias is spelled incorrectly for this handler?")
-        if $!;
-	
-	
+	if($_[HEAP]->{'LOGHANDLER'}) {
+		$! = undef;
+		$_[KERNEL]->post(
+		   $_[HEAP]->{'LOGHANDLER'}->{'SESSION'},
+		   $_[HEAP]->{'LOGHANDLER'}->{'EVENT'},
+		   $request,
+		   $response->connection->remote_ip());
+
+		# Warn if we had a problem dispatching to the log handler above
+		warn("I had a problem posting to event '",
+		     $_[HEAP]->{'LOGHANDLER'}->{'EVENT'},
+		     "' of the log handler alias '",
+		     $_[HEAP]->{'LOGHANDLE'}->{'SESSION'},
+		  "'. As reported by Kernel: '$!', perhaps the alias is spelled incorrectly for this handler?")
+		if $!;
+	}
+
+
 	# If we received a malformed request then
 	# let's not try to dispatch to a handler
 	if( $malformed_req ) {
-        # Just push out the response we got from POE::Filter::HTTPD saying your request was bad
-        $_[KERNEL]->yield('DONE', $response);
-	} else {
+		# Just push out the response we got from POE::Filter::HTTPD saying your request was bad
+		POE::Kernel->post(
+			$_[HEAP]->{ERRORHANDLER}->{SESSION},
+			$_[HEAP]->{ERRORHANDLER}->{EVENT},
+			'BadRequest (by POE::Filter::HTTPD)', $response->connection->remote_ip()
+		);
+		$_[KERNEL]->yield('DONE', $response);
+		return;
+	} 
+	else {
 	   # Find which handler will handle this one
 		foreach my $handler ( @{ $_[HEAP]->{'HANDLERS'} } ) {
 			# Check if this matches
@@ -750,7 +836,7 @@ sub Got_Input {
 				croak("I had a problem posting to event $handler->{'EVENT'} of session $handler->{'SESSION'} for DIR handler '$handler->{'DIR'}'",
 			      	". As reported by Kernel: '$!', perhaps the session name is spelled incorrectly for this handler?")
                 	if $!;
-	
+
 				# All done!
 				return;
 			}
@@ -762,6 +848,7 @@ sub Got_Input {
 		$response->content('404 Not Found');
         $_[KERNEL]->yield('DONE', $response);
 	}
+	return;
 }
 
 # Finished with a request!
@@ -773,7 +860,7 @@ sub Got_Flush {
 	if ( DEBUG ) {
 		warn "Got Flush event for wheel ID ( $id )";
 	}
-	
+
    if ($_[HEAP]->{'REQUESTS'}->{ $id }->[1] == 2 ) {
 		# Do the stream !
 		if ( DEBUG ) {
@@ -783,14 +870,26 @@ sub Got_Flush {
 	}
 	# Check if we are shutting down
 	elsif ( $_[HEAP]->{'REQUESTS'}->{ $id }->[1] == 1 ) {
-		# Shutdown read/write on the wheel
-		$_[HEAP]->{'REQUESTS'}->{ $id }->[0]->shutdown_input();
-		$_[HEAP]->{'REQUESTS'}->{ $id }->[0]->shutdown_output();
 
-		# Delete the wheel
-		# Tracked down by Paul Visscher
+        if( Must_KeepAlive( $_[HEAP], $id ) ) {
+            if ( DEBUG ) {
+                warn "Keep-alive id=$id ...";
+            }
+            $_[HEAP]->{'CONNECTIONS'}->{ $id } = [
+                      $_[HEAP]->{'REQUESTS'}->{ $id }->[0],   # wheel
+                      $_[HEAP]->{'REQUESTS'}->{ $id }->[2]->connection
+                    ];
+        }
+        else {
+    		# Shutdown read/write on the wheel
+	    	$_[HEAP]->{'REQUESTS'}->{ $id }->[0]->shutdown_input();
+		    $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->shutdown_output();
+        }
+
+        # Delete the wheel
+	    # Tracked down by Paul Visscher
 		delete $_[HEAP]->{'REQUESTS'}->{ $id }->[0];
-		delete $_[HEAP]->{'REQUESTS'}->{ $id };
+    	delete $_[HEAP]->{'REQUESTS'}->{ $id };
 	}
 	else {
 		# Ignore this, eh?
@@ -802,7 +901,8 @@ sub Got_Flush {
 	# Alright, do we have to shutdown?
 	if ( ! exists $_[HEAP]->{'SOCKETFACTORY'} ) {
 		# Check to see if we have any more requests
-		if ( keys( %{ $_[HEAP]->{'REQUESTS'} } ) == 0 ) {
+		if ( keys( %{ $_[HEAP]->{'REQUESTS'} } ) == 0 and
+             keys( %{ $_[HEAP]->{'CONNECTIONS'} } ) == 0 ) {
 			# Shutdown!
 			$_[KERNEL]->yield( 'SHUTDOWN' );
 		}
@@ -812,27 +912,62 @@ sub Got_Flush {
 	return 1;
 }
 
+
+# should we keep-alive the connection?
+sub Must_KeepAlive
+{
+    my( $heap, $id ) = @_;
+
+    return unless $heap->{'KEEPALIVE'};
+
+    my $resp = $heap->{'REQUESTS'}->{ $id }->[2];
+    my $req  = $heap->{'REQUESTS'}->{ $id }->[3];
+
+    # error = close
+    return 0 if $resp->is_error;
+
+    # Connection is a comma-seperated header
+    my $conn = lc $req->header( 'Connection' );
+    return 0 if ",$conn," =~ /,\s*close\s*,/;
+    $conn = lc $resp->header( 'Connection' );
+    return 0 if ",$conn," =~ /,\s*close\s*,/;
+
+    # HTTP/1.1 = keep
+    return 1 if $req->protocol eq 'HTTP/1.1';
+    return 0;
+}
+
 # Got some sort of error from ReadWrite
 sub Got_Error {
 	# ARG0 = operation, ARG1 = error number, ARG2 = error string, ARG3 = wheel ID
-	my ( $operation, $errnum, $errstr, $id ) = @_[ ARG0 .. ARG3 ];
+   my ( $operation, $errnum, $errstr, $id ) = @_[ ARG0 .. ARG3 ];
 
 	# Only do this for non-EOF on read
-	unless ( $operation eq 'read' and $errnum == 0 ) {
+   unless ( $operation eq 'read' and $errnum == 0 ) {
 		# Debug stuff
-		if ( DEBUG ) {
-			warn "Wheel $id generated $operation error $errnum: $errstr\n";
-		}
-
-		# Make the client dead
-		$_[HEAP]->{'REQUESTS'}->{ $id }->[2]->{'CONNECTION'}->{'DIED'} = 1;
-
-		# Delete this connection
-		delete $_[HEAP]->{'REQUESTS'}->{ $id };
+	if ( DEBUG ) {
+  	   warn "Wheel $id generated $operation error $errnum: $errstr\n";
 	}
 
+        my $connection;
+        if( $_[HEAP]->{'CONNECTIONS'}{ $id } ) {
+            my $c = delete $_[HEAP]->{'CONNECTIONS'}{ $id };
+            $connection = $c->[1];
+            delete $c->[0];
+        }
+        else {
+            $connection = $_[HEAP]->{'REQUESTS'}->{ $id }->[2]->{'CONNECTION'};
+    		# Delete this connection
+	    	delete $_[HEAP]->{'REQUESTS'}->{ $id }->[0];
+	    	delete $_[HEAP]->{'REQUESTS'}->{ $id };
+        }
+
+		# Mark the client dead
+		$connection->{'DIED'} = 1;
+   }
+
 	# Success!
-	return 1;
+   return 1;
 }
 
 # Output to the client!
@@ -859,12 +994,12 @@ sub Request_Output {
 		if ( DEBUG ) {
 			warn 'Wheel disappeared, but the application sent us a DONE event, discarding it';
 		}
-		
-      POE::Kernel->post(
-         $_[HEAP]->{ERRORHANDLER}->{SESSION},
-         $_[HEAP]->{ERRORHANDLER}->{EVENT},
-         'Wheel disappeared !'
-      );
+
+      		POE::Kernel->post(
+         		$_[HEAP]->{ERRORHANDLER}->{SESSION},
+         		$_[HEAP]->{ERRORHANDLER}->{EVENT},
+         		'Wheel disappeared !'
+      		);
 		# All done!
 		return 1;
 	}
@@ -881,41 +1016,46 @@ sub Request_Output {
 		if ( DEBUG ) {
 			warn 'Tried to send data over a closed/nonexistant socket!';
 		}
-      POE::Kernel->post(
-         $_[HEAP]->{ERRORHANDLER}->{SESSION},
-         $_[HEAP]->{ERRORHANDLER}->{EVENT},
-         'Socket closed/nonexistant !'
-      ); 
+      		POE::Kernel->post(
+         		$_[HEAP]->{ERRORHANDLER}->{SESSION},
+         		$_[HEAP]->{ERRORHANDLER}->{EVENT},
+         		'Socket closed/nonexistant !'
+      		);
 		return;
 	}
 
-	# Set the date if needed
-	if ( ! $response->header( 'Date' ) ) {
-		$response->header( 'Date', time2str( time ) );
-	}
+    Fix_Headers( $_[HEAP], $response );
 
-	# Set the Content-Length if needed
-	if ( ! $response->header( 'Content-Length' ) ) {
-		use bytes;
-		$response->header( 'Content-Length', length( $response->content ) );
-	}
-
-	# Set the Content-Type if needed
-	if ( ! $response->header( 'Content-Type' ) ) {
-		$response->header( 'Content-Type', 'text/html' );
-	}
-   
    # Send it out!
    $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->put( $response );
 
    # Mark this socket done
    $_[HEAP]->{'REQUESTS'}->{ $id }->[1] = 1;
 
+	# Log FINALLY If they have a logFinal handler registered, send out the needed information
+	if($_[HEAP]->{'LOG2HANDLER'}) {
+		$! = undef;
+		$_[KERNEL]->call(
+			$_[HEAP]->{'LOG2HANDLER'}->{'SESSION'},
+			$_[HEAP]->{'LOG2HANDLER'}->{'EVENT'},
+			$_[HEAP]->{'REQUESTS'}{ $id }[3],
+			$response);
+
+		# Warn if we had a problem dispatching to the log handler above
+		warn("I had a problem posting to event '",
+			$_[HEAP]->{'LOG2HANDLER'}->{'EVENT'},
+			"' of the log handler alias '",
+			$_[HEAP]->{'LOG2HANDLER'}->{'SESSION'},
+			"'. As reported by Kernel: '$!', perhaps the alias is spelled incorrectly for this handler?")
+		if $!;
+	}
+
+
    # Debug stuff
    if ( DEBUG ) {
            warn "Completed with Wheel ID $id";
    }
-  
+
    # Success!
    return 1;
 }
@@ -923,24 +1063,24 @@ sub Request_Output {
 # Stream output to the client
 sub Stream_Output {
    # ARG0 = HTTP::Response object
-   my ($kernel, $response) = @_[KERNEL, ARG0 ];   
-   
+   my ($kernel, $response) = @_[KERNEL, ARG0 ];
+
    # Check if we got it
    if ( ! defined $response or ! UNIVERSAL::isa( $response, 'HTTP::Response' ) ) {
       if ( DEBUG ) {
          warn 'Did not get a HTTP::Response object!';
       }
-      
+
       # Abort...
       return undef;
    }
-   
+
    # Get the wheel ID
    my $id = $response->_WHEEL;
    $_[HEAP]->{'CHUNKCOUNT'}->{ $id }++;
 
    if (defined $response->{'STREAM'}) {
-      # Keep track if we plan to stream ...   	
+      # Keep track if we plan to stream ...
       if ( $_[HEAP]->{'RESPONSES'}->{ $id } ) {
          if ( DEBUG ) {
             warn "Restoring response from HEAP and id $id ";
@@ -972,14 +1112,14 @@ sub Stream_Output {
          $_[HEAP]->{ERRORHANDLER}->{SESSION},
          $_[HEAP]->{ERRORHANDLER}->{EVENT},
          'Wheel disappeared !'
-      ); 
+      );
       # All done!
       return 1;
    }
-   
+
    # Quick check to see if the wheel/socket died already...
    # Initially reported by Tim Wood
-   if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $id }->[0] 
+   if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $id }->[0]
       or ! defined $_[HEAP]->{'REQUESTS'}->{ $id }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
       if ( DEBUG ) {
          warn 'Tried to send data over a closed/nonexistant socket!';
@@ -988,22 +1128,14 @@ sub Stream_Output {
          $_[HEAP]->{ERRORHANDLER}->{SESSION},
          $_[HEAP]->{ERRORHANDLER}->{EVENT},
          'Socket closed/nonexistant !'
-      ); 
+      );
       return;
    }
-   
-   # Set the date if needed
-   if ( ! $response->header( 'Date' ) ) {
-      $response->header( 'Date', time2str( time ) );
-   }
-   
-   # Set the Content-Type if needed
-   if ( ! $response->header( 'Content-Type' ) ) {
-      $response->header( 'Content-Type', 'text/html' );
-   }
+
+   Fix_Headers( $_[HEAP], $response, 1 );
 
    # Preliminary check
-   if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $response->_WHEEL }->[0] 
+   if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $response->_WHEEL }->[0]
       or ! defined $_[HEAP]->{'REQUESTS'}->{ $response->_WHEEL }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
       if ( DEBUG ) {
          warn 'Tried to send data over a closed/nonexistant socket!';
@@ -1015,14 +1147,14 @@ sub Stream_Output {
    unless (defined $response->{'IS_STREAMING'}) {
       # Mark this socket done
       $_[HEAP]->{'REQUESTS'}->{ $id }->[1] = 2;
-      
+
       #
       $response->{'IS_STREAMING'} = 1;
    }
-   
+
    if ( DEBUG ) {
       warn "Sending stream via ".$response->{STREAM_SESSION}."/".$response->{STREAM}." with id $id \n" ;
-   }      
+   }
 
    if ( $_[HEAP]->{'CHUNKCOUNT'}->{ $id } > 1  ) {
       $_[HEAP]->{'REQUESTS'}->{ $response->_WHEEL }->[0]->set_output_filter(POE::Filter::Stream->new() ) ;
@@ -1034,21 +1166,51 @@ sub Stream_Output {
    }
 
    $response->content();
-   
-   # we send the event to stream with wheels request and response to the session 
-   # that has registered the streaming event     
-   unless ($response->{'dont_flush'}) {          
+
+   # we send the event to stream with wheels request and response to the session
+   # that has registered the streaming event
+   unless ($response->{'DONT_FLUSH'}) {
       POE::Kernel->post(
          $response->{STREAM_SESSION},           # callback session
          $response->{STREAM},                   # callback event
          $_[HEAP]->{'RESPONSES'}->{$response->_WHEEL}
-      ); 
+      );
    }
-	
+
     # Success!
 	return 1;
 }
 
+# Add required headers to a response
+sub Fix_Headers
+{
+   my( $heap, $response, $stream ) = @_;
+    # Set the date if needed
+    if ( ! $response->header( 'Date' ) ) {
+        $response->header( 'Date', time2str( time ) );
+    }
+
+	# Set the Content-Length if needed
+	if ( !$stream and ! defined $response->header( 'Content-Length' ) and my $len = length $response->content ) {
+		use bytes;
+		$response->header( 'Content-Length', $len );
+	}
+
+	# Set the Content-Type if needed
+	if ( ! $response->header( 'Content-Type' ) ) {
+		$response->header( 'Content-Type', 'text/plain' );
+	}
+
+    if( ! $response->protocol ) {
+        my $request = $heap->{'REQUESTS'}->{ $response->_WHEEL }->[3];
+	return unless $request and $request->isa('HTTP::Request');
+        unless( $request->method eq 'HEAD' ) {
+            $response->protocol( $request->protocol );
+        }
+    }
+}
+
+
 # Closes the connection
 sub Request_Close {
 	# ARG0 = HTTP::Response object
@@ -1067,6 +1229,12 @@ sub Request_Close {
 	# Get the wheel ID
 	my $id = $response->_WHEEL;
 
+    if( $_[HEAP]->{'CONNECTIONS'}->{ $id } ) {
+        my $c = delete $_[HEAP]->{'CONNECTIONS'}->{ $id };
+        $_[HEAP]->{'REQUESTS'}->{ $id } = [ $c->[0], 0, undef ];
+    }
+
+
 	# Check if the wheel exists ( sometimes it gets closed by the client, but the application doesn't know that... )
 	if ( ! exists $_[HEAP]->{'REQUESTS'}->{ $id } ) {
 		# Debug stuff
@@ -1101,14 +1269,45 @@ sub Request_Close {
 # Registers a POE inline state (primarly for streaming)
 sub Register {
    my ( $session , $state, $code_ref) = @_[SESSION, ARG0 .. ARG1];
-   
+
 	if ( DEBUG ) {
 		warn 'Registering state in POE session';
 	}
-   
+
    return $session->register_state( $state, $code_ref );
 }
 
+# SETCLOSEHANDLER
+sub SetCloseHandler
+{
+    my( $heap, $sender ) = @_[ HEAP, SENDER ];
+    my( $connection, $state, @params ) = @_[ ARG0..$#_ ];
+
+    # turn connection ID into the connection object
+    unless( ref $connection ) {
+        my $id = $connection;
+        if( $heap->{'CONNECTIONS'}->{$id} ) {
+            $connection = $heap->{'CONNECTIONS'}->{$id}->[1];
+        }
+        elsif( $heap->{'REQUESTS'}->{$id} and
+                $heap->{'REQUESTS'}->{$id}->[2] ) {
+            $connection = $_[HEAP]->{'REQUESTS'}->{ $id }->[2]->connection;
+        }
+        unless( ref $connection ) {
+            die "Can't find connection object for request $id";
+        }
+    }
+
+    if( $state ) {
+        $connection->_on_close( $sender, $state, @params );
+    }
+    else {
+        $connection->_on_close( $sender );
+    }
+}
+
+
+
 # End of module
 1;
 
@@ -1155,6 +1354,10 @@ POE::Component::Server::SimpleHTTP - Perl extension to serve HTTP requests in PO
 				  'EVENT'   => 'GOT_LOG',
 		},
 
+		'LOG2HANDLER' => { 'SESSION' => 'HTTP_GET',
+				  'EVENT'   => 'POSTLOG',
+		},
+
 		# In the testing phase...
 		'SSLKEYCERT'	=>	[ 'public-key.pem', 'public-cert.pem' ],
 	) or die 'Unable to create the HTTP Server';
@@ -1230,7 +1433,7 @@ POE::Component::Server::SimpleHTTP - Perl extension to serve HTTP requests in PO
 	}
 
 	sub GOT_LOG {
-		# ARG0 = HTTP::Request object, ARG1 = remote IP 
+		# ARG0 = HTTP::Request object, ARG1 = remote IP
 		my ($request, $remote_ip) = @_[ARG0,ARG1];
 
 		# Do some sort of logging activity.
@@ -1355,6 +1558,16 @@ is a "catch-all" DIR regex like '.*', it will catch the errors, and only that on
 NOTE: The only way SimpleHTTP will leak memory ( hopefully heh ) is if you discard the SimpleHTTP::Response object without sending it
 back to SimpleHTTP via the DONE/CLOSE events, so never do that!
 
+=item C<KEEPALIVE>
+
+Set to true to enable HTTP keep-alive support.  Connections will be
+kept alive until the client closes the connection.  All HTTP/1.1 connections
+are kept-open, unless you set the response C<Connection> header to C<close>.
+
+    $response->header( Connection => 'close' );
+
+If you want more control, use L<POE::Component::Server::HTTP::KeepAlive>.
+
 =item C<LOGHANDLER>
 
 Expects a hashref with the following key, values:
@@ -1365,6 +1578,7 @@ EVENT	->	The event to trigger
 
 You will receive an event for each request to the server from clients.  Malformed client requests will not be passed into the handler.  Instead
 undef will be passed.
+Event is called before ANY content handler is called.
 
 The event will have the following parameters:
 
@@ -1372,6 +1586,33 @@ ARG0 -> HTTP::Request object/undef if client request was malformed.
 
 ARG1 -> the IP address of the client
 
+=item C<LOG2HANDLER>
+
+Expect a hashref with the following key, valyes:
+
+SESSION	->	The session to send the input
+
+EVENT	->	The event to trigger
+
+You will receive an event for each response that hit DONE call. Malformed client requests will not be passed into the handler.
+Event is after processing all content handlers.
+
+The event will have the following parameters:
+
+ARG0 -> HTTP::Request object
+
+ARG1 -> HTTP::Response object
+
+That makes possible following code:
+
+	my ($login, $password) = $request->authorization_basic();
+	printf STDERR "%s - %s [%s] \"%s %s %s\" %d %d\n",
+		$response->connection->remote_ip, $login||'-', POSIX::strftime("%d/%b/%Y:%T %z",localtime(time())),
+		$request->method(), $request->uri()->path(), $request->protocol(),
+		$response->code(), length($response->content());
+
+Emulate apache-like logs for PoCo::Server::SimpleHTTP
+
 =item C<SETUPHANDLER>
 
 Expects a hashref with the following key, values:
@@ -1391,6 +1632,11 @@ is greatly welcome!
 
 Again, this will automatically turn every incoming connection into a SSL socket. Once enough testing has been done, this option will be augmented with more SSL stuff!
 
+=item C<PROXYMODE>
+
+Set this to a true value to enable the server to act as a proxy server, ie. it won't mangle the HTTP::Request
+URI.
+
 =back
 
 =head2 Events
@@ -1433,6 +1679,25 @@ SimpleHTTP is so simple, there are only 8 events available.
 
 	BEWARE: if there is an error in the HANDLERS, SimpleHTTP will die!
 
+=item C<SETCLOSEHANDLER>
+
+    $_[KERNEL]->call( $_[SENDER], 'SETCLOSEHANDLER', $connection,
+                      $event, @args );
+
+Calls C<$event> in the current session when C<$connection> is closed.  You
+could use for persistent connection handling.
+
+Multiple session may register close handlers.
+
+Calling SETCLOSEHANDLER without C<$event> to remove the current session's
+handler:
+
+   $_[KERNEL]->call( $_[SENDER], 'SETCLOSEHANDLER', $connection );
+
+You B<must> make sure that C<@args> doesn't cause a circular
+reference.  Ideally, use C<$connection->ID> or some other unique value
+associated with this C<$connection>.
+
 =item C<STARTLISTEN>
 
 	Starts the listening socket, if it was shut down
@@ -1456,15 +1721,15 @@ SimpleHTTP is so simple, there are only 8 events available.
 
 	With a $response argument it streams the content and calls back the streaming event
 	of the user's session (or with the dont_flush option you're responsible for calling
-        back your session's streaming event).	
-	
+        back your session's streaming event).
+
 	To use the streaming feature see below.
 
 =back
 
 =head2 Streaming with SimpleHTTP
 
-It's possible to send data as a stream to clients (unbuffered and integrated in the 
+It's possible to send data as a stream to clients (unbuffered and integrated in the
 POE loop).
 
 Just create your session to receive events from SimpleHTTP as usually and add a
@@ -1478,7 +1743,7 @@ $response to a streaming state and once you trigger it:
       event       => 'GOT_STREAM',
       dont_flush  => 1
    );
-   
+
    # then you can simply yield your streaming event, once the GOT_STREAM event
    # has reached its end it will be triggered again and again, until you
    # send a CLOSE event to the kernel with the appropriate response as parameter
@@ -1493,10 +1758,10 @@ shutdown when your streaming is done (EOF for example).
 
    sub GOT_STREAM {
       my ( $kernel, $heap, $response ) = @_[KERNEL, HEAP, ARG0];
-      
+
       # sets the content of the response
       $response->content("Hello World\n");
-      
+
       # send it to the client
       POE::Kernel->post('HTTPD', 'STREAM', $response);
 
@@ -1516,14 +1781,10 @@ shutdown when your streaming is done (EOF for example).
    }
 
 The dont_flush option is there to be able to control the frequency of flushes
-to the client. 
+to the client.
 
 =head2 SimpleHTTP Notes
 
-This module is very picky about capitalization!
-
-All of the options are uppercase, to avoid confusion.
-
 You can enable debugging mode by doing this:
 
 	sub POE::Component::Server::SimpleHTTP::DEBUG () { 1 }
